#!/usr/bin/env python
# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.


"""
Messages relating to signatures
"""

import json

import construct

from sbp.msg import SBP, SENDER_ID
from sbp.utils import fmt_repr, exclude_fields, walk_json_dict, containerize

# Automatically generated from piksi/yaml/swiftnav/sbp/signing.yaml with generate.py.
# Please do not hand edit!


SBP_MSG_ED25519_SIGNATURE = 0x0C01
class MsgEd25519Signature(SBP):
  """SBP class for message MSG_ED25519_SIGNATURE (0x0C01).

  You can have MSG_ED25519_SIGNATURE inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  signature : array
    ED25519 signature for messages.
  fingerprint : array
    SHA-1 fingerprint of the associated certificate.
  signed_messages : array
    CRCs of signed messages.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'signature' / construct.Array(64, construct.Int8ul),
                   'fingerprint' / construct.Array(20, construct.Int8ul),
                   'signed_messages' / construct.GreedyRange(construct.Int32ul),)
  __slots__ = [
               'signature',
               'fingerprint',
               'signed_messages',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgEd25519Signature,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgEd25519Signature, self).__init__()
      self.msg_type = SBP_MSG_ED25519_SIGNATURE
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.signature = kwargs.pop('signature')
      self.fingerprint = kwargs.pop('fingerprint')
      self.signed_messages = kwargs.pop('signed_messages')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgEd25519Signature.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgEd25519Signature(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgEd25519Signature._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgEd25519Signature._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgEd25519Signature._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgEd25519Signature, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_ED25519_CERTIFICATE = 0x0C02
class MsgEd25519Certificate(SBP):
  """SBP class for message MSG_ED25519_CERTIFICATE (0x0C02).

  You can have MSG_ED25519_CERTIFICATE inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  n_msg : int
    Total number messages that make up the certificate. First nibble is the
    size of the sequence (n), second nibble is the zero-indexed counter (ith
    packet of n)
  fingerprint : array
    SHA-1 fingerprint of the associated certificate.
  certificate_bytes : array
    ED25519 certificate bytes.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'n_msg' / construct.Int8ul,
                   'fingerprint' / construct.Array(20, construct.Int8ul),
                   'certificate_bytes' / construct.GreedyRange(construct.Int8ul),)
  __slots__ = [
               'n_msg',
               'fingerprint',
               'certificate_bytes',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgEd25519Certificate,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgEd25519Certificate, self).__init__()
      self.msg_type = SBP_MSG_ED25519_CERTIFICATE
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.n_msg = kwargs.pop('n_msg')
      self.fingerprint = kwargs.pop('fingerprint')
      self.certificate_bytes = kwargs.pop('certificate_bytes')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgEd25519Certificate.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgEd25519Certificate(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgEd25519Certificate._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgEd25519Certificate._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgEd25519Certificate._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgEd25519Certificate, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    

msg_classes = {
  0x0C01: MsgEd25519Signature,
  0x0C02: MsgEd25519Certificate,
}


# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
"""
The :mod:`sbp.client.handler` module contains classes related to
SBP message handling.
"""

import warnings
import collections
import threading
import weakref
import six
from six.moves.queue import Queue

try:
    from typing import Optional  # noqa
except ImportError:
    pass


class Handler(object):
    """
    Handler

    The :class:`Handler` class provides an interface for connecting handlers
    to a driver providing SBP messages.  Also provides queued and filtered
    iterators for synchronous, blocking use in other threads.

    Parameters
    ----------
    source : Iterable of tuple(SBP message, {'time':'ISO 8601 str'})
      Stream of SBP messages
    autostart : Boolean
      If false, start() shall be skipped when entering context scope and it
      should be explicitly called by the parent. This will prevent losing
      messages in case where receive thread would otherwise be started before
      consumers are ready.
    """

    def __init__(self, source, autostart=True):
        self._autostart = autostart
        self._source = source
        self._callbacks = collections.defaultdict(set)
        self._receive_thread = threading.Thread(
            target=self._recv_thread, name="Handler")
        self._receive_thread.daemon = True
        self._sinks = []  # This is a list of weakrefs to upstream iterators
        self._dead = False
        self._exception = None
        self._write_lock = threading.Lock()

    def _recv_thread(self):
        """
        Internal thread to iterate over source messages and dispatch callbacks.
        """
        def gen_messages():
            for msg, metadata in self._source:
                if msg.msg_type:
                    yield (msg, metadata)

        messages = gen_messages()
        while True:
            msg_and_metadata = None
            try:
                msg_and_metadata = next(messages, None)
            except Exception as exc:
                self._exception = exc
                break
            if msg_and_metadata is None:
                break
            msg, metadata = msg_and_metadata
            self._call(msg, **metadata)
        # Break any upstream iterators
        for sink in self._sinks:
            i = sink()
            if i is not None:
                i.breakiter(self._exception)
        self._dead = True

    def __enter__(self):
        if self._autostart:
            self.start()
        return self

    def __exit__(self, *args):
        self.stop()

    # This exception is raised when a message is dispatched to a garbage
    # collected upstream iterator.
    class _DeadCallbackException(Exception):
        pass

    def filter(self, msg_type=None, maxsize=0):
        """
        Get a filtered iterator of messages for synchronous, blocking use in
        another thread.
        """
        if self._dead:
            return iter(())
        iterator = Handler._SBPQueueIterator(maxsize)
        # We use a weakref so that the iterator may be garbage collected if it's
        # consumer no longer has a reference.
        ref = weakref.ref(iterator)
        self._sinks.append(ref)

        def feediter(msg, **metadata):
            i = ref()
            if i is not None:
                i(msg, **metadata)
            else:
                raise Handler._DeadCallbackException

        self.add_callback(feediter, msg_type)
        return iterator

    def __iter__(self):
        """
        Get a queued iterator that will provide the same unfiltered messages
        read from the source iterator.
        """
        return self.filter()

    def _to_iter(self, maybe_iter):
        try:
            return iter(maybe_iter)
        except TypeError:
            return None

    def add_callback(self, callback, msg_type=None):
        """
        Add per message type or global callback.

        Parameters
        ----------
        callback : fn
          Callback function
        msg_type : int | iterable
          Message type to register callback against. Default `None` means global callback.
          Iterable type adds the callback to all the message types.
        """
        cb_keys = self._to_iter(msg_type)
        if cb_keys is not None:
            for msg_type_ in cb_keys:
                self._callbacks[msg_type_].add(callback)
        else:
            self._callbacks[msg_type].add(callback)

    def remove_callback(self, callback, msg_type=None):
        """
        Remove per message type of global callback.

        Parameters
        ----------
        callback : fn
          Callback function
        msg_type : int | iterable
          Message type to remove callback from. Default `None` means global callback.
          Iterable type removes the callback from all the message types.
        """
        if msg_type is None:
            msg_type = self._callbacks.keys()
        cb_keys = self._to_iter(msg_type)
        if cb_keys is not None:
            for msg_type_ in cb_keys:
                try:
                    self._callbacks[msg_type_].remove(callback)
                except KeyError:
                    pass
        else:
            self._callbacks[msg_type].remove(callback)

    def _gc_dead_sinks(self):
        """
        Remove any dead weakrefs.
        """
        deadsinks = []
        for i in self._sinks:
            if i() is None:
                deadsinks.append(i)
        for i in deadsinks:
            self._sinks.remove(i)

    def _get_callbacks(self, msg_type):
        """
        Return all callbacks (global and per message type) for a message type.

        Parameters
        ----------
        msg_type : int
          Message type to return callbacks for.
        """
        return self._callbacks[None] | self._callbacks[msg_type]

    def _call(self, msg, **metadata):
        """
        Process message with all callbacks (global and per message type).
        """
        if msg.msg_type:
            for callback in self._get_callbacks(msg.msg_type):
                try:
                    callback(msg, **metadata)
                except Handler._DeadCallbackException:
                    # The callback was an upstream iterator that has been garbage
                    # collected.  Remove it from our internal structures.
                    self.remove_callback(callback)
                    self._gc_dead_sinks()
                except SystemExit:
                    raise
                except:
                    import traceback
                    traceback.print_exc()

    def start(self):
        """
        Start processing SBP messages with handlers.
        """
        self._receive_thread.start()

    def stop(self):
        """
        Stop processing SBP messages.
        """
        try:
            self._source.breakiter()
            self._receive_thread.join(0.1)
        except Exception as exc:
            warnings.warn("Handler stop error: %s" % (exc,))

    def join(self, timeout=None):
        self._receive_thread.join(timeout)

    def is_alive(self):
        """
        Return whether the processes thread is alive.
        """
        return self._receive_thread.is_alive()

    def wait(self, msg_type, timeout=1.0):
        """
        Wait for a SBP message.

        Parameters
        ----------
        msg_type : int
          SBP message type.
        timeout : float
          Waiting period
        """
        event = threading.Event()
        payload = {'data': None}

        def cb(sbp_msg, **metadata):
            payload['data'] = sbp_msg
            event.set()

        self.add_callback(cb, msg_type)
        event.wait(timeout)
        self.remove_callback(cb, msg_type)
        return payload['data']

    def wait_callback(self, callback, msg_type=None, timeout=1.0):
        """
        Wait for a SBP message with a callback.

        Parameters
        ----------
        callback : fn
          Callback function
        msg_type : int | iterable
          Message type to register callback against. Default `None` means global callback.
          Iterable type adds the callback to all the message types.
        timeout : float
          Waiting period
        """
        event = threading.Event()

        def cb(msg, **metadata):
            callback(msg, **metadata)
            event.set()

        self.add_callback(cb, msg_type)
        event.wait(timeout)
        self.remove_callback(cb, msg_type)

    def __call__(self, *msgs, **metadata):
        """
        Pass messages to the `source` to be consumed.  Typically this means
        the messages will be framed and transmitted via whatever transport
        layer is currently active.

        Parameters
        ----------
        msgs : SBP messages
          SBP messages to send.
        metadata : dict
          Metadata for this batch of messages, passed to the `source`.
        """
        with self._write_lock:
            self._source(*msgs, **metadata)

    class _SBPQueueIterator(six.Iterator):
        """
        Class for upstream iterators.  Implements callable interface for adding
        messages into the queue, and iterable interface for getting them out.
        """

        def __init__(self, maxsize):
            self._queue = Queue(maxsize)
            self._broken = False
            self._exception = None  # type: Optional[Exception]

        def __iter__(self):
            return self

        def __call__(self, msg, **metadata):
            self._queue.put((msg, metadata), False)

        def breakiter(self, exc=None):
            self._broken = True
            self._exception = exc
            self._queue.put(None, True, 1.0)

        def __next__(self):
            if self._broken and self._queue.empty():
                raise StopIteration
            m = self._queue.get(True)
            if self._broken and m is None:
                if self._exception is not None:
                    raise self._exception
                raise StopIteration
            return m



from __future__ import print_function

import sys

import io

import json

from construct.core import StreamError

import sbp.msg
import sbp.table

from sbp.msg import SBP_PREAMBLE
from sbp.msg import UnpackError


DEFAULT_JSON='rapidjson'
JSON_CHOICES=['json', 'rapidjson']
try:
    import rapidjson
except ImportError:
    DEFAULT_JSON='json'
    JSON_CHOICES=['json']

try:
    memoryview
except NameError:
    memoryview = lambda x: x


def base_cl_options():
    import argparse
    parser = argparse.ArgumentParser(prog="sbp2json", description="Swift Navigation SBP to JSON parser")
    parser.add_argument('--mode', type=str, choices=JSON_CHOICES, default=DEFAULT_JSON,
                        help="the JSON serialization library to use, default: {}".format(DEFAULT_JSON))
    parser.add_argument('--include', nargs="+", type=int, default=[],
                        help="list of SBP message IDs to include, empty means all")
    parser.add_argument('file', nargs='?', metavar='FILE', type=argparse.FileType('rb'),
                        default=sys.stdin, help="the input file, stdin by default")

    group_json = parser.add_argument_group('json specific arguments')
    group_json.add_argument(
        "--sort-keys",
        action="store_true",
        help="Sort JSON log elements by keys")

    return parser


def get_args():
    """
    Get and parse arguments.
    """
    parser = base_cl_options()
    args = parser.parse_args()

    return args


def dump(args, res):
    if 'json' == args.mode:
        sys.stdout.write(json.dumps(res.to_json_dict(),
                                    allow_nan=False,
                                    sort_keys=args.sort_keys,
                                    separators=(',', ':')))
    elif 'rapidjson' == args.mode:
        sys.stdout.write(rapidjson.dumps(res.to_json_dict()))
    sys.stdout.write("\n")


def sbp_main(args):
    header_len = 6
    reader = io.open(args.file.fileno(), 'rb')
    buf = memoryview(bytearray(4096))
    unconsumed_offset = 0
    read_offset = 0
    buffer_remaining = len(buf)
    include = set(args.include)
    while True:
        if buffer_remaining == 0:
            buf[0:(read_offset - unconsumed_offset)] = buf[unconsumed_offset:read_offset]
            read_offset = read_offset - unconsumed_offset
            unconsumed_offset = 0
            buffer_remaining = len(buf) - read_offset
        mv = buf[read_offset:]
        read_length = reader.readinto(mv)
        if read_length == 0:
            unconsumed = read_offset - unconsumed_offset
            if unconsumed != 0:
                sys.stderr.write("unconsumed: {}\n".format(unconsumed))
                sys.stderr.flush()
            break
        read_offset += read_length
        buffer_remaining -= read_length
        while True:
            bytes_available = read_offset - unconsumed_offset
            b = buf[unconsumed_offset:(unconsumed_offset + bytes_available)]
            if len(b) == 0:
                break
            if b[0] != SBP_PREAMBLE:
                consumed = 1
            else:
                try:
                    m = sbp.msg.SBP.unpack(b)
                    if not include or m.msg_type in include:
                        m = sbp.table.dispatch(m)
                        dump(args, m)
                    consumed = header_len + m.length + 2
                except (UnpackError, StreamError, ValueError):
                    consumed = 1
            unconsumed_offset += consumed


def module_main():
    args = get_args()
    if not args:
        sys.exit(1)
    try:
        sbp_main(args)
    except KeyboardInterrupt:
        pass



# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.


class NullLogger(object):
    """
    NullLogger

    The :class:`NullLogger` does not log records.
    """

    def __call__(self, msg, **metadata):
        pass

    def __enter__(self):
        return self

    def flush(self):
        pass

    def read(self, s=None):
        pass

    def write(self, s=None):
        pass

    def close(self):
        pass

    def __exit__(self, *args):
        pass



#!/usr/bin/env python
# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.


"""
SBAS data
"""

import json

import construct

from sbp.msg import SBP, SENDER_ID
from sbp.utils import fmt_repr, exclude_fields, walk_json_dict, containerize
from sbp.gnss import GnssSignal

# Automatically generated from piksi/yaml/swiftnav/sbp/sbas.yaml with generate.py.
# Please do not hand edit!


SBP_MSG_SBAS_RAW = 0x7777
class MsgSbasRaw(SBP):
  """SBP class for message MSG_SBAS_RAW (0x7777).

  You can have MSG_SBAS_RAW inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This message is sent once per second per SBAS satellite. ME checks the
  parity of the data block and sends only blocks that pass the check.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  sid : GnssSignal
    GNSS signal identifier.
  tow : int
    GPS time-of-week at the start of the data block.
  message_type : int
    SBAS message type (0-63)
  data : array
    Raw SBAS data field of 212 bits (last byte padded with zeros).
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'sid' / GnssSignal._parser,
                   'tow' / construct.Int32ul,
                   'message_type' / construct.Int8ul,
                   'data' / construct.Array(27, construct.Int8ul),)
  __slots__ = [
               'sid',
               'tow',
               'message_type',
               'data',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgSbasRaw,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgSbasRaw, self).__init__()
      self.msg_type = SBP_MSG_SBAS_RAW
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.sid = kwargs.pop('sid')
      self.tow = kwargs.pop('tow')
      self.message_type = kwargs.pop('message_type')
      self.data = kwargs.pop('data')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgSbasRaw.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgSbasRaw(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgSbasRaw._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgSbasRaw._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgSbasRaw._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgSbasRaw, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    

msg_classes = {
  0x7777: MsgSbasRaw,
}


#!/usr/bin/env python
# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.


"""
Messages for logging NDB events.
"""

import json

import construct

from sbp.msg import SBP, SENDER_ID
from sbp.utils import fmt_repr, exclude_fields, walk_json_dict, containerize
from sbp.gnss import GnssSignal

# Automatically generated from piksi/yaml/swiftnav/sbp/ndb.yaml with generate.py.
# Please do not hand edit!


SBP_MSG_NDB_EVENT = 0x0400
class MsgNdbEvent(SBP):
  """SBP class for message MSG_NDB_EVENT (0x0400).

  You can have MSG_NDB_EVENT inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This message is sent out when an object is stored into NDB. If needed
  message could also be sent out when fetching an object from NDB.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  recv_time : int
    HW time in milliseconds.
  event : int
    Event type.
  object_type : int
    Event object type.
  result : int
    Event result.
  data_source : int
    Data source for STORE event, reserved for other events.
  object_sid : GnssSignal
    GNSS signal identifier, If object_type is Ephemeris OR Almanac, sid
    indicates for which signal the object belongs to. Reserved in other cases.
  src_sid : GnssSignal
    GNSS signal identifier, If object_type is Almanac, Almanac WN, Iono OR L2C
    capabilities AND data_source is NDB_DS_RECEIVER sid indicates from which
    SV data was decoded. Reserved in other cases.
  original_sender : int
    A unique identifier of the sending hardware. For v1.0, set to the 2 least
    significant bytes of the device serial number, valid only if data_source
    is NDB_DS_SBP. Reserved in case of other data_source.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'recv_time' / construct.Int64ul,
                   'event' / construct.Int8ul,
                   'object_type' / construct.Int8ul,
                   'result' / construct.Int8ul,
                   'data_source' / construct.Int8ul,
                   'object_sid' / GnssSignal._parser,
                   'src_sid' / GnssSignal._parser,
                   'original_sender' / construct.Int16ul,)
  __slots__ = [
               'recv_time',
               'event',
               'object_type',
               'result',
               'data_source',
               'object_sid',
               'src_sid',
               'original_sender',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgNdbEvent,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgNdbEvent, self).__init__()
      self.msg_type = SBP_MSG_NDB_EVENT
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.recv_time = kwargs.pop('recv_time')
      self.event = kwargs.pop('event')
      self.object_type = kwargs.pop('object_type')
      self.result = kwargs.pop('result')
      self.data_source = kwargs.pop('data_source')
      self.object_sid = kwargs.pop('object_sid')
      self.src_sid = kwargs.pop('src_sid')
      self.original_sender = kwargs.pop('original_sender')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgNdbEvent.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgNdbEvent(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgNdbEvent._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgNdbEvent._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgNdbEvent._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgNdbEvent, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    

msg_classes = {
  0x0400: MsgNdbEvent,
}


#!/usr/bin/env python
# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.


"""
Linux state monitoring.
"""

import json

import construct

from sbp.msg import SBP, SENDER_ID
from sbp.utils import fmt_repr, exclude_fields, walk_json_dict, containerize

# Automatically generated from piksi/yaml/swiftnav/sbp/linux.yaml with generate.py.
# Please do not hand edit!


SBP_MSG_LINUX_CPU_STATE_DEP_A = 0x7F00
class MsgLinuxCpuStateDepA(SBP):
  """SBP class for message MSG_LINUX_CPU_STATE_DEP_A (0x7F00).

  You can have MSG_LINUX_CPU_STATE_DEP_A inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This message indicates the process state of the top 10 heaviest consumers of
  CPU on the system.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  index : int
    sequence of this status message, values from 0-9
  pid : int
    the PID of the process
  pcpu : int
    percent of cpu used, expressed as a fraction of 256
  tname : string
    fixed length string representing the thread name
  cmdline : string
    the command line (as much as it fits in the remaining packet)
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'index' / construct.Int8ul,
                   'pid' / construct.Int16ul,
                   'pcpu' / construct.Int8ul,
                   'tname'/ construct.Bytes(15),
                   'cmdline' / construct.GreedyBytes,)
  __slots__ = [
               'index',
               'pid',
               'pcpu',
               'tname',
               'cmdline',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgLinuxCpuStateDepA,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgLinuxCpuStateDepA, self).__init__()
      self.msg_type = SBP_MSG_LINUX_CPU_STATE_DEP_A
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.index = kwargs.pop('index')
      self.pid = kwargs.pop('pid')
      self.pcpu = kwargs.pop('pcpu')
      self.tname = kwargs.pop('tname')
      self.cmdline = kwargs.pop('cmdline')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgLinuxCpuStateDepA.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgLinuxCpuStateDepA(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgLinuxCpuStateDepA._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgLinuxCpuStateDepA._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgLinuxCpuStateDepA._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgLinuxCpuStateDepA, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_LINUX_MEM_STATE_DEP_A = 0x7F01
class MsgLinuxMemStateDepA(SBP):
  """SBP class for message MSG_LINUX_MEM_STATE_DEP_A (0x7F01).

  You can have MSG_LINUX_MEM_STATE_DEP_A inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This message indicates the process state of the top 10 heaviest consumers of
  memory on the system.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  index : int
    sequence of this status message, values from 0-9
  pid : int
    the PID of the process
  pmem : int
    percent of memory used, expressed as a fraction of 256
  tname : string
    fixed length string representing the thread name
  cmdline : string
    the command line (as much as it fits in the remaining packet)
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'index' / construct.Int8ul,
                   'pid' / construct.Int16ul,
                   'pmem' / construct.Int8ul,
                   'tname'/ construct.Bytes(15),
                   'cmdline' / construct.GreedyBytes,)
  __slots__ = [
               'index',
               'pid',
               'pmem',
               'tname',
               'cmdline',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgLinuxMemStateDepA,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgLinuxMemStateDepA, self).__init__()
      self.msg_type = SBP_MSG_LINUX_MEM_STATE_DEP_A
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.index = kwargs.pop('index')
      self.pid = kwargs.pop('pid')
      self.pmem = kwargs.pop('pmem')
      self.tname = kwargs.pop('tname')
      self.cmdline = kwargs.pop('cmdline')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgLinuxMemStateDepA.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgLinuxMemStateDepA(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgLinuxMemStateDepA._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgLinuxMemStateDepA._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgLinuxMemStateDepA._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgLinuxMemStateDepA, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_LINUX_SYS_STATE_DEP_A = 0x7F02
class MsgLinuxSysStateDepA(SBP):
  """SBP class for message MSG_LINUX_SYS_STATE_DEP_A (0x7F02).

  You can have MSG_LINUX_SYS_STATE_DEP_A inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This presents a summary of CPU and memory utilization.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  mem_total : int
    total system memory
  pcpu : int
    percent of total cpu currently utilized
  pmem : int
    percent of total memory currently utilized
  procs_starting : int
    number of processes that started during collection phase
  procs_stopping : int
    number of processes that stopped during collection phase
  pid_count : int
    the count of processes on the system
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'mem_total' / construct.Int16ul,
                   'pcpu' / construct.Int8ul,
                   'pmem' / construct.Int8ul,
                   'procs_starting' / construct.Int16ul,
                   'procs_stopping' / construct.Int16ul,
                   'pid_count' / construct.Int16ul,)
  __slots__ = [
               'mem_total',
               'pcpu',
               'pmem',
               'procs_starting',
               'procs_stopping',
               'pid_count',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgLinuxSysStateDepA,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgLinuxSysStateDepA, self).__init__()
      self.msg_type = SBP_MSG_LINUX_SYS_STATE_DEP_A
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.mem_total = kwargs.pop('mem_total')
      self.pcpu = kwargs.pop('pcpu')
      self.pmem = kwargs.pop('pmem')
      self.procs_starting = kwargs.pop('procs_starting')
      self.procs_stopping = kwargs.pop('procs_stopping')
      self.pid_count = kwargs.pop('pid_count')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgLinuxSysStateDepA.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgLinuxSysStateDepA(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgLinuxSysStateDepA._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgLinuxSysStateDepA._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgLinuxSysStateDepA._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgLinuxSysStateDepA, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_LINUX_PROCESS_SOCKET_COUNTS = 0x7F03
class MsgLinuxProcessSocketCounts(SBP):
  """SBP class for message MSG_LINUX_PROCESS_SOCKET_COUNTS (0x7F03).

  You can have MSG_LINUX_PROCESS_SOCKET_COUNTS inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Top 10 list of processes with high socket counts.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  index : int
    sequence of this status message, values from 0-9
  pid : int
    the PID of the process in question
  socket_count : int
    the number of sockets the process is using
  socket_types : int
    A bitfield indicating the socket types used: 0x1 (tcp), 0x2 (udp), 0x4
    (unix stream), 0x8 (unix dgram), 0x10 (netlink), and 0x8000 (unknown)
  socket_states : int
    A bitfield indicating the socket states: 0x1 (established), 0x2 (syn-
    sent), 0x4 (syn-recv), 0x8 (fin-wait-1), 0x10 (fin-wait-2), 0x20 (time-
    wait), 0x40 (closed), 0x80 (close-wait), 0x100 (last-ack), 0x200 (listen),
    0x400 (closing), 0x800 (unconnected), and 0x8000 (unknown)
  cmdline : string
    the command line of the process in question
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'index' / construct.Int8ul,
                   'pid' / construct.Int16ul,
                   'socket_count' / construct.Int16ul,
                   'socket_types' / construct.Int16ul,
                   'socket_states' / construct.Int16ul,
                   'cmdline' / construct.GreedyBytes,)
  __slots__ = [
               'index',
               'pid',
               'socket_count',
               'socket_types',
               'socket_states',
               'cmdline',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgLinuxProcessSocketCounts,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgLinuxProcessSocketCounts, self).__init__()
      self.msg_type = SBP_MSG_LINUX_PROCESS_SOCKET_COUNTS
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.index = kwargs.pop('index')
      self.pid = kwargs.pop('pid')
      self.socket_count = kwargs.pop('socket_count')
      self.socket_types = kwargs.pop('socket_types')
      self.socket_states = kwargs.pop('socket_states')
      self.cmdline = kwargs.pop('cmdline')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgLinuxProcessSocketCounts.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgLinuxProcessSocketCounts(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgLinuxProcessSocketCounts._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgLinuxProcessSocketCounts._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgLinuxProcessSocketCounts._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgLinuxProcessSocketCounts, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_LINUX_PROCESS_SOCKET_QUEUES = 0x7F04
class MsgLinuxProcessSocketQueues(SBP):
  """SBP class for message MSG_LINUX_PROCESS_SOCKET_QUEUES (0x7F04).

  You can have MSG_LINUX_PROCESS_SOCKET_QUEUES inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Top 10 list of sockets with deep queues.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  index : int
    sequence of this status message, values from 0-9
  pid : int
    the PID of the process in question
  recv_queued : int
    the total amount of receive data queued for this process
  send_queued : int
    the total amount of send data queued for this process
  socket_types : int
    A bitfield indicating the socket types used: 0x1 (tcp), 0x2 (udp), 0x4
    (unix stream), 0x8 (unix dgram), 0x10 (netlink), and 0x8000 (unknown)
  socket_states : int
    A bitfield indicating the socket states: 0x1 (established), 0x2 (syn-
    sent), 0x4 (syn-recv), 0x8 (fin-wait-1), 0x10 (fin-wait-2), 0x20 (time-
    wait), 0x40 (closed), 0x80 (close-wait), 0x100 (last-ack), 0x200 (listen),
    0x400 (closing), 0x800 (unconnected), and 0x8000 (unknown)
  address_of_largest : string
    Address of the largest queue, remote or local depending on the
    directionality of the connection.
  cmdline : string
    the command line of the process in question
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'index' / construct.Int8ul,
                   'pid' / construct.Int16ul,
                   'recv_queued' / construct.Int16ul,
                   'send_queued' / construct.Int16ul,
                   'socket_types' / construct.Int16ul,
                   'socket_states' / construct.Int16ul,
                   'address_of_largest'/ construct.Bytes(64),
                   'cmdline' / construct.GreedyBytes,)
  __slots__ = [
               'index',
               'pid',
               'recv_queued',
               'send_queued',
               'socket_types',
               'socket_states',
               'address_of_largest',
               'cmdline',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgLinuxProcessSocketQueues,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgLinuxProcessSocketQueues, self).__init__()
      self.msg_type = SBP_MSG_LINUX_PROCESS_SOCKET_QUEUES
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.index = kwargs.pop('index')
      self.pid = kwargs.pop('pid')
      self.recv_queued = kwargs.pop('recv_queued')
      self.send_queued = kwargs.pop('send_queued')
      self.socket_types = kwargs.pop('socket_types')
      self.socket_states = kwargs.pop('socket_states')
      self.address_of_largest = kwargs.pop('address_of_largest')
      self.cmdline = kwargs.pop('cmdline')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgLinuxProcessSocketQueues.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgLinuxProcessSocketQueues(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgLinuxProcessSocketQueues._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgLinuxProcessSocketQueues._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgLinuxProcessSocketQueues._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgLinuxProcessSocketQueues, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_LINUX_SOCKET_USAGE = 0x7F05
class MsgLinuxSocketUsage(SBP):
  """SBP class for message MSG_LINUX_SOCKET_USAGE (0x7F05).

  You can have MSG_LINUX_SOCKET_USAGE inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Summaries the socket usage across the system.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  avg_queue_depth : int
    average socket queue depths across all sockets on the system
  max_queue_depth : int
    the max queue depth seen within the reporting period
  socket_state_counts : array
    A count for each socket type reported in the `socket_types_reported`
    field, the first entry corresponds to the first enabled bit in
    `types_reported`.
  socket_type_counts : array
    A count for each socket type reported in the `socket_types_reported`
    field, the first entry corresponds to the first enabled bit in
    `types_reported`.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'avg_queue_depth' / construct.Int32ul,
                   'max_queue_depth' / construct.Int32ul,
                   'socket_state_counts' / construct.Array(16, construct.Int16ul),
                   'socket_type_counts' / construct.Array(16, construct.Int16ul),)
  __slots__ = [
               'avg_queue_depth',
               'max_queue_depth',
               'socket_state_counts',
               'socket_type_counts',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgLinuxSocketUsage,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgLinuxSocketUsage, self).__init__()
      self.msg_type = SBP_MSG_LINUX_SOCKET_USAGE
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.avg_queue_depth = kwargs.pop('avg_queue_depth')
      self.max_queue_depth = kwargs.pop('max_queue_depth')
      self.socket_state_counts = kwargs.pop('socket_state_counts')
      self.socket_type_counts = kwargs.pop('socket_type_counts')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgLinuxSocketUsage.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgLinuxSocketUsage(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgLinuxSocketUsage._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgLinuxSocketUsage._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgLinuxSocketUsage._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgLinuxSocketUsage, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_LINUX_PROCESS_FD_COUNT = 0x7F06
class MsgLinuxProcessFdCount(SBP):
  """SBP class for message MSG_LINUX_PROCESS_FD_COUNT (0x7F06).

  You can have MSG_LINUX_PROCESS_FD_COUNT inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Top 10 list of processes with a large number of open file descriptors.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  index : int
    sequence of this status message, values from 0-9
  pid : int
    the PID of the process in question
  fd_count : int
    a count of the number of file descriptors opened by the process
  cmdline : string
    the command line of the process in question
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'index' / construct.Int8ul,
                   'pid' / construct.Int16ul,
                   'fd_count' / construct.Int16ul,
                   'cmdline' / construct.GreedyBytes,)
  __slots__ = [
               'index',
               'pid',
               'fd_count',
               'cmdline',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgLinuxProcessFdCount,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgLinuxProcessFdCount, self).__init__()
      self.msg_type = SBP_MSG_LINUX_PROCESS_FD_COUNT
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.index = kwargs.pop('index')
      self.pid = kwargs.pop('pid')
      self.fd_count = kwargs.pop('fd_count')
      self.cmdline = kwargs.pop('cmdline')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgLinuxProcessFdCount.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgLinuxProcessFdCount(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgLinuxProcessFdCount._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgLinuxProcessFdCount._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgLinuxProcessFdCount._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgLinuxProcessFdCount, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_LINUX_PROCESS_FD_SUMMARY = 0x7F07
class MsgLinuxProcessFdSummary(SBP):
  r"""SBP class for message MSG_LINUX_PROCESS_FD_SUMMARY (0x7F07).

  You can have MSG_LINUX_PROCESS_FD_SUMMARY inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Summary of open file descriptors on the system.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  sys_fd_count : int
    count of total FDs open on the system
  most_opened : string
    A null delimited list of strings which alternates between a string
    representation of the process count and the file name whose count it being
    reported.  That is, in C string syntax
    "32\0/var/log/syslog\012\0/tmp/foo\0" with the end of the list being 2
    NULL terminators in a row.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'sys_fd_count' / construct.Int32ul,
                   'most_opened' / construct.GreedyBytes,)
  __slots__ = [
               'sys_fd_count',
               'most_opened',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgLinuxProcessFdSummary,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgLinuxProcessFdSummary, self).__init__()
      self.msg_type = SBP_MSG_LINUX_PROCESS_FD_SUMMARY
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.sys_fd_count = kwargs.pop('sys_fd_count')
      self.most_opened = kwargs.pop('most_opened')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgLinuxProcessFdSummary.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgLinuxProcessFdSummary(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgLinuxProcessFdSummary._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgLinuxProcessFdSummary._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgLinuxProcessFdSummary._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgLinuxProcessFdSummary, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_LINUX_CPU_STATE = 0x7F08
class MsgLinuxCpuState(SBP):
  """SBP class for message MSG_LINUX_CPU_STATE (0x7F08).

  You can have MSG_LINUX_CPU_STATE inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This message indicates the process state of the top 10 heaviest consumers of
  CPU on the system, including a timestamp.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  index : int
    sequence of this status message, values from 0-9
  pid : int
    the PID of the process
  pcpu : int
    percent of CPU used, expressed as a fraction of 256
  time : int
    timestamp of message, refer to flags field for how to interpret
  flags : int
    flags
  tname : string
    fixed length string representing the thread name
  cmdline : string
    the command line (as much as it fits in the remaining packet)
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'index' / construct.Int8ul,
                   'pid' / construct.Int16ul,
                   'pcpu' / construct.Int8ul,
                   'time' / construct.Int32ul,
                   'flags' / construct.Int8ul,
                   'tname'/ construct.Bytes(15),
                   'cmdline' / construct.GreedyBytes,)
  __slots__ = [
               'index',
               'pid',
               'pcpu',
               'time',
               'flags',
               'tname',
               'cmdline',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgLinuxCpuState,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgLinuxCpuState, self).__init__()
      self.msg_type = SBP_MSG_LINUX_CPU_STATE
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.index = kwargs.pop('index')
      self.pid = kwargs.pop('pid')
      self.pcpu = kwargs.pop('pcpu')
      self.time = kwargs.pop('time')
      self.flags = kwargs.pop('flags')
      self.tname = kwargs.pop('tname')
      self.cmdline = kwargs.pop('cmdline')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgLinuxCpuState.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgLinuxCpuState(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgLinuxCpuState._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgLinuxCpuState._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgLinuxCpuState._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgLinuxCpuState, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_LINUX_MEM_STATE = 0x7F09
class MsgLinuxMemState(SBP):
  """SBP class for message MSG_LINUX_MEM_STATE (0x7F09).

  You can have MSG_LINUX_MEM_STATE inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This message indicates the process state of the top 10 heaviest consumers of
  memory on the system, including a timestamp.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  index : int
    sequence of this status message, values from 0-9
  pid : int
    the PID of the process
  pmem : int
    percent of memory used, expressed as a fraction of 256
  time : int
    timestamp of message, refer to flags field for how to interpret
  flags : int
    flags
  tname : string
    fixed length string representing the thread name
  cmdline : string
    the command line (as much as it fits in the remaining packet)
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'index' / construct.Int8ul,
                   'pid' / construct.Int16ul,
                   'pmem' / construct.Int8ul,
                   'time' / construct.Int32ul,
                   'flags' / construct.Int8ul,
                   'tname'/ construct.Bytes(15),
                   'cmdline' / construct.GreedyBytes,)
  __slots__ = [
               'index',
               'pid',
               'pmem',
               'time',
               'flags',
               'tname',
               'cmdline',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgLinuxMemState,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgLinuxMemState, self).__init__()
      self.msg_type = SBP_MSG_LINUX_MEM_STATE
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.index = kwargs.pop('index')
      self.pid = kwargs.pop('pid')
      self.pmem = kwargs.pop('pmem')
      self.time = kwargs.pop('time')
      self.flags = kwargs.pop('flags')
      self.tname = kwargs.pop('tname')
      self.cmdline = kwargs.pop('cmdline')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgLinuxMemState.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgLinuxMemState(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgLinuxMemState._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgLinuxMemState._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgLinuxMemState._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgLinuxMemState, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_LINUX_SYS_STATE = 0x7F0A
class MsgLinuxSysState(SBP):
  """SBP class for message MSG_LINUX_SYS_STATE (0x7F0A).

  You can have MSG_LINUX_SYS_STATE inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This presents a summary of CPU and memory utilization, including a
  timestamp.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  mem_total : int
    total system memory, in MiB
  pcpu : int
    percent of CPU used, expressed as a fraction of 256
  pmem : int
    percent of memory used, expressed as a fraction of 256
  procs_starting : int
    number of processes that started during collection phase
  procs_stopping : int
    number of processes that stopped during collection phase
  pid_count : int
    the count of processes on the system
  time : int
    timestamp of message, refer to flags field for how to interpret
  flags : int
    flags
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'mem_total' / construct.Int16ul,
                   'pcpu' / construct.Int8ul,
                   'pmem' / construct.Int8ul,
                   'procs_starting' / construct.Int16ul,
                   'procs_stopping' / construct.Int16ul,
                   'pid_count' / construct.Int16ul,
                   'time' / construct.Int32ul,
                   'flags' / construct.Int8ul,)
  __slots__ = [
               'mem_total',
               'pcpu',
               'pmem',
               'procs_starting',
               'procs_stopping',
               'pid_count',
               'time',
               'flags',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgLinuxSysState,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgLinuxSysState, self).__init__()
      self.msg_type = SBP_MSG_LINUX_SYS_STATE
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.mem_total = kwargs.pop('mem_total')
      self.pcpu = kwargs.pop('pcpu')
      self.pmem = kwargs.pop('pmem')
      self.procs_starting = kwargs.pop('procs_starting')
      self.procs_stopping = kwargs.pop('procs_stopping')
      self.pid_count = kwargs.pop('pid_count')
      self.time = kwargs.pop('time')
      self.flags = kwargs.pop('flags')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgLinuxSysState.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgLinuxSysState(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgLinuxSysState._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgLinuxSysState._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgLinuxSysState._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgLinuxSysState, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    

msg_classes = {
  0x7F00: MsgLinuxCpuStateDepA,
  0x7F01: MsgLinuxMemStateDepA,
  0x7F02: MsgLinuxSysStateDepA,
  0x7F03: MsgLinuxProcessSocketCounts,
  0x7F04: MsgLinuxProcessSocketQueues,
  0x7F05: MsgLinuxSocketUsage,
  0x7F06: MsgLinuxProcessFdCount,
  0x7F07: MsgLinuxProcessFdSummary,
  0x7F08: MsgLinuxCpuState,
  0x7F09: MsgLinuxMemState,
  0x7F0A: MsgLinuxSysState,
}


#!/usr/bin/env python
# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.


"""
Messages for reading/writing the device's onboard flash memory. Many of
these messages target specific flash memory peripherals used in Swift
Navigation devices: the STM32 flash and the M25Pxx FPGA configuration flash
from Piksi 2.3.1.  This module does not apply to Piksi Multi.
"""

import json

import construct

from sbp.msg import SBP, SENDER_ID
from sbp.utils import fmt_repr, exclude_fields, walk_json_dict, containerize

# Automatically generated from piksi/yaml/swiftnav/sbp/flash.yaml with generate.py.
# Please do not hand edit!


SBP_MSG_FLASH_PROGRAM = 0x00E6
class MsgFlashProgram(SBP):
  """SBP class for message MSG_FLASH_PROGRAM (0x00E6).

  You can have MSG_FLASH_PROGRAM inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The flash program message programs a set of addresses of either the STM or
  M25 flash. The device replies with either a MSG_FLASH_DONE message
  containing the return code FLASH_OK (0) on success, or FLASH_INVALID_LEN (2)
  if the maximum write size is exceeded. Note that the sector-containing
  addresses must be erased before addresses can be programmed.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  target : int
    Target flags
  addr_start : array
    Starting address offset to program
  addr_len : int
    Length of set of addresses to program, counting up from starting address
  data : array
    Data to program addresses with, with length N=addr_len
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'target' / construct.Int8ul,
                   'addr_start' / construct.Array(3, construct.Int8ul),
                   'addr_len' / construct.Int8ul,
                   'data' / construct.GreedyRange(construct.Int8ul),)
  __slots__ = [
               'target',
               'addr_start',
               'addr_len',
               'data',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgFlashProgram,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgFlashProgram, self).__init__()
      self.msg_type = SBP_MSG_FLASH_PROGRAM
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.target = kwargs.pop('target')
      self.addr_start = kwargs.pop('addr_start')
      self.addr_len = kwargs.pop('addr_len')
      self.data = kwargs.pop('data')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgFlashProgram.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgFlashProgram(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgFlashProgram._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgFlashProgram._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgFlashProgram._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgFlashProgram, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_FLASH_DONE = 0x00E0
class MsgFlashDone(SBP):
  """SBP class for message MSG_FLASH_DONE (0x00E0).

  You can have MSG_FLASH_DONE inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This message defines success or failure codes for a variety of flash memory
  requests from the host to the device. Flash read and write messages, such as
  MSG_FLASH_READ_REQ, or MSG_FLASH_PROGRAM, may return this message on
  failure.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  response : int
    Response flags
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'response' / construct.Int8ul,)
  __slots__ = [
               'response',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgFlashDone,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgFlashDone, self).__init__()
      self.msg_type = SBP_MSG_FLASH_DONE
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.response = kwargs.pop('response')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgFlashDone.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgFlashDone(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgFlashDone._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgFlashDone._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgFlashDone._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgFlashDone, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_FLASH_READ_REQ = 0x00E7
class MsgFlashReadReq(SBP):
  """SBP class for message MSG_FLASH_READ_REQ (0x00E7).

  You can have MSG_FLASH_READ_REQ inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The flash read message reads a set of addresses of either the STM or M25
  onboard flash. The device replies with a MSG_FLASH_READ_RESP message
  containing either the read data on success or a MSG_FLASH_DONE message
  containing the return code FLASH_INVALID_LEN (2) if the maximum read size is
  exceeded or FLASH_INVALID_ADDR (3) if the address is outside of the allowed
  range.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  target : int
    Target flags
  addr_start : array
    Starting address offset to read from
  addr_len : int
    Length of set of addresses to read, counting up from starting address
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'target' / construct.Int8ul,
                   'addr_start' / construct.Array(3, construct.Int8ul),
                   'addr_len' / construct.Int8ul,)
  __slots__ = [
               'target',
               'addr_start',
               'addr_len',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgFlashReadReq,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgFlashReadReq, self).__init__()
      self.msg_type = SBP_MSG_FLASH_READ_REQ
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.target = kwargs.pop('target')
      self.addr_start = kwargs.pop('addr_start')
      self.addr_len = kwargs.pop('addr_len')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgFlashReadReq.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgFlashReadReq(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgFlashReadReq._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgFlashReadReq._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgFlashReadReq._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgFlashReadReq, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_FLASH_READ_RESP = 0x00E1
class MsgFlashReadResp(SBP):
  """SBP class for message MSG_FLASH_READ_RESP (0x00E1).

  You can have MSG_FLASH_READ_RESP inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The flash read message reads a set of addresses of either the STM or M25
  onboard flash. The device replies with a MSG_FLASH_READ_RESP message
  containing either the read data on success or a MSG_FLASH_DONE message
  containing the return code FLASH_INVALID_LEN (2) if the maximum read size is
  exceeded or FLASH_INVALID_ADDR (3) if the address is outside of the allowed
  range.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  target : int
    Target flags
  addr_start : array
    Starting address offset to read from
  addr_len : int
    Length of set of addresses to read, counting up from starting address
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'target' / construct.Int8ul,
                   'addr_start' / construct.Array(3, construct.Int8ul),
                   'addr_len' / construct.Int8ul,)
  __slots__ = [
               'target',
               'addr_start',
               'addr_len',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgFlashReadResp,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgFlashReadResp, self).__init__()
      self.msg_type = SBP_MSG_FLASH_READ_RESP
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.target = kwargs.pop('target')
      self.addr_start = kwargs.pop('addr_start')
      self.addr_len = kwargs.pop('addr_len')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgFlashReadResp.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgFlashReadResp(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgFlashReadResp._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgFlashReadResp._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgFlashReadResp._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgFlashReadResp, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_FLASH_ERASE = 0x00E2
class MsgFlashErase(SBP):
  """SBP class for message MSG_FLASH_ERASE (0x00E2).

  You can have MSG_FLASH_ERASE inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The flash erase message from the host erases a sector of either the STM or
  M25 onboard flash memory. The device will reply with a MSG_FLASH_DONE
  message containing the return code - FLASH_OK (0) on success or
  FLASH_INVALID_FLASH (1) if the flash specified is invalid.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  target : int
    Target flags
  sector_num : int
    Flash sector number to erase (0-11 for the STM, 0-15 for the M25)
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'target' / construct.Int8ul,
                   'sector_num' / construct.Int32ul,)
  __slots__ = [
               'target',
               'sector_num',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgFlashErase,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgFlashErase, self).__init__()
      self.msg_type = SBP_MSG_FLASH_ERASE
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.target = kwargs.pop('target')
      self.sector_num = kwargs.pop('sector_num')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgFlashErase.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgFlashErase(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgFlashErase._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgFlashErase._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgFlashErase._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgFlashErase, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_STM_FLASH_LOCK_SECTOR = 0x00E3
class MsgStmFlashLockSector(SBP):
  """SBP class for message MSG_STM_FLASH_LOCK_SECTOR (0x00E3).

  You can have MSG_STM_FLASH_LOCK_SECTOR inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The flash lock message locks a sector of the STM flash memory. The device
  replies with a MSG_FLASH_DONE message.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  sector : int
    Flash sector number to lock
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'sector' / construct.Int32ul,)
  __slots__ = [
               'sector',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgStmFlashLockSector,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgStmFlashLockSector, self).__init__()
      self.msg_type = SBP_MSG_STM_FLASH_LOCK_SECTOR
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.sector = kwargs.pop('sector')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgStmFlashLockSector.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgStmFlashLockSector(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgStmFlashLockSector._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgStmFlashLockSector._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgStmFlashLockSector._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgStmFlashLockSector, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_STM_FLASH_UNLOCK_SECTOR = 0x00E4
class MsgStmFlashUnlockSector(SBP):
  """SBP class for message MSG_STM_FLASH_UNLOCK_SECTOR (0x00E4).

  You can have MSG_STM_FLASH_UNLOCK_SECTOR inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The flash unlock message unlocks a sector of the STM flash memory. The
  device replies with a MSG_FLASH_DONE message.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  sector : int
    Flash sector number to unlock
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'sector' / construct.Int32ul,)
  __slots__ = [
               'sector',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgStmFlashUnlockSector,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgStmFlashUnlockSector, self).__init__()
      self.msg_type = SBP_MSG_STM_FLASH_UNLOCK_SECTOR
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.sector = kwargs.pop('sector')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgStmFlashUnlockSector.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgStmFlashUnlockSector(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgStmFlashUnlockSector._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgStmFlashUnlockSector._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgStmFlashUnlockSector._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgStmFlashUnlockSector, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_STM_UNIQUE_ID_REQ = 0x00E8
class MsgStmUniqueIdReq(SBP):
  """SBP class for message MSG_STM_UNIQUE_ID_REQ (0x00E8).

  You can have MSG_STM_UNIQUE_ID_REQ inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This message reads the device's hard-coded unique ID. The host requests the
  ID by sending a MSG_STM_UNIQUE_ID_REQ. The device responds with a
  MSG_STM_UNIQUE_ID_RESP with the 12-byte unique ID in the payload.

  """
  __slots__ = []

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgStmUniqueIdReq,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.payload = sbp.payload
    else:
      super( MsgStmUniqueIdReq, self).__init__()
      self.msg_type = SBP_MSG_STM_UNIQUE_ID_REQ
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.payload = b""

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgStmUniqueIdReq.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgStmUniqueIdReq(sbp, **d)

 
    
SBP_MSG_STM_UNIQUE_ID_RESP = 0x00E5
class MsgStmUniqueIdResp(SBP):
  """SBP class for message MSG_STM_UNIQUE_ID_RESP (0x00E5).

  You can have MSG_STM_UNIQUE_ID_RESP inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This message reads the device's hard-coded unique ID. The host requests the
  ID by sending a MSG_STM_UNIQUE_ID_REQ. The device responds with a
  MSG_STM_UNIQUE_ID_RESP with the 12-byte unique ID in the payload.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  stm_id : array
    Device unique ID
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'stm_id' / construct.Array(12, construct.Int8ul),)
  __slots__ = [
               'stm_id',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgStmUniqueIdResp,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgStmUniqueIdResp, self).__init__()
      self.msg_type = SBP_MSG_STM_UNIQUE_ID_RESP
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.stm_id = kwargs.pop('stm_id')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgStmUniqueIdResp.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgStmUniqueIdResp(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgStmUniqueIdResp._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgStmUniqueIdResp._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgStmUniqueIdResp._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgStmUniqueIdResp, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_M25_FLASH_WRITE_STATUS = 0x00F3
class MsgM25FlashWriteStatus(SBP):
  """SBP class for message MSG_M25_FLASH_WRITE_STATUS (0x00F3).

  You can have MSG_M25_FLASH_WRITE_STATUS inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The flash status message writes to the 8-bit M25 flash status register. The
  device replies with a MSG_FLASH_DONE message.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  status : array
    Byte to write to the M25 flash status register
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'status' / construct.Array(1, construct.Int8ul),)
  __slots__ = [
               'status',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgM25FlashWriteStatus,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgM25FlashWriteStatus, self).__init__()
      self.msg_type = SBP_MSG_M25_FLASH_WRITE_STATUS
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.status = kwargs.pop('status')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgM25FlashWriteStatus.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgM25FlashWriteStatus(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgM25FlashWriteStatus._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgM25FlashWriteStatus._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgM25FlashWriteStatus._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgM25FlashWriteStatus, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    

msg_classes = {
  0x00E6: MsgFlashProgram,
  0x00E0: MsgFlashDone,
  0x00E7: MsgFlashReadReq,
  0x00E1: MsgFlashReadResp,
  0x00E2: MsgFlashErase,
  0x00E3: MsgStmFlashLockSector,
  0x00E4: MsgStmFlashUnlockSector,
  0x00E8: MsgStmUniqueIdReq,
  0x00E5: MsgStmUniqueIdResp,
  0x00F3: MsgM25FlashWriteStatus,
}


#!/usr/bin/env python
# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.


"""
Various structs shared between modules
"""

import construct

from sbp.utils import fmt_repr

# Automatically generated from piksi/yaml/swiftnav/sbp/gnss.yaml with generate.py.
# Please do not hand edit!


class GnssSignal(object):
  """GnssSignal.
  
  Signal identifier containing constellation, band, and satellite identifier.
  
  Parameters
  ----------
  sat : int
    Constellation-specific satellite id. For GLO can either be (100+FCN) where
    FCN is in [-7,+6] or the Slot ID in [1,28].
  code : int
    Signal constellation, band and code

  """
  _parser = construct.Struct(
                     'sat' / construct.Int8ul,
                     'code' / construct.Int8ul,)
  __slots__ = [
               'sat',
               'code',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.sat = kwargs.pop('sat')
      self.code = kwargs.pop('code')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = GnssSignal._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
class SvId(object):
  """SvId.
  
  A (Constellation ID, satellite ID) tuple that uniquely identifies a space
  vehicle.
  
  Parameters
  ----------
  satId : int
    Constellation-specific satellite id. For GLO can either be (100+FCN) where
    FCN is in [-7,+6] or the Slot ID in [1,28].
  constellation : int
    Constellation ID to which the SV belongs

  """
  _parser = construct.Struct(
                     'satId' / construct.Int8ul,
                     'constellation' / construct.Int8ul,)
  __slots__ = [
               'satId',
               'constellation',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.satId = kwargs.pop('satId')
      self.constellation = kwargs.pop('constellation')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = SvId._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
class GnssSignalDep(object):
  """GnssSignalDep.
  
  Deprecated.
  
  Parameters
  ----------
  sat : int
    Constellation-specific satellite identifier.

    Note: unlike GnssSignal, GPS satellites are encoded as (PRN - 1). Other
    constellations do not have this offset.
  code : int
    Signal constellation, band and code
  reserved : int
    Reserved

  """
  _parser = construct.Struct(
                     'sat' / construct.Int16ul,
                     'code' / construct.Int8ul,
                     'reserved' / construct.Int8ul,)
  __slots__ = [
               'sat',
               'code',
               'reserved',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.sat = kwargs.pop('sat')
      self.code = kwargs.pop('code')
      self.reserved = kwargs.pop('reserved')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = GnssSignalDep._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
class GPSTimeDep(object):
  """GPSTimeDep.
  
  A wire-appropriate GPS time, defined as the number of milliseconds since
  beginning of the week on the Saturday/Sunday transition.
  
  Parameters
  ----------
  tow : int
    Milliseconds since start of GPS week
  wn : int
    GPS week number

  """
  _parser = construct.Struct(
                     'tow' / construct.Int32ul,
                     'wn' / construct.Int16ul,)
  __slots__ = [
               'tow',
               'wn',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.tow = kwargs.pop('tow')
      self.wn = kwargs.pop('wn')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = GPSTimeDep._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
class GPSTimeSec(object):
  """GPSTimeSec.
  
  A GPS time, defined as the number of seconds since beginning of the week on
  the Saturday/Sunday transition.
  
  Parameters
  ----------
  tow : int
    Seconds since start of GPS week
  wn : int
    GPS week number

  """
  _parser = construct.Struct(
                     'tow' / construct.Int32ul,
                     'wn' / construct.Int16ul,)
  __slots__ = [
               'tow',
               'wn',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.tow = kwargs.pop('tow')
      self.wn = kwargs.pop('wn')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = GPSTimeSec._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
class GPSTime(object):
  """GPSTime.
  
  A wire-appropriate receiver clock time, defined as the time since the
  beginning of the week on the Saturday/Sunday transition. In most cases,
  observations are epoch aligned so ns field will be 0.
  
  Parameters
  ----------
  tow : int
    Milliseconds since start of GPS week
  ns_residual : int
    Nanosecond residual of millisecond-rounded TOW (ranges from -500000 to
    500000)
  wn : int
    GPS week number

  """
  _parser = construct.Struct(
                     'tow' / construct.Int32ul,
                     'ns_residual' / construct.Int32sl,
                     'wn' / construct.Int16ul,)
  __slots__ = [
               'tow',
               'ns_residual',
               'wn',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.tow = kwargs.pop('tow')
      self.ns_residual = kwargs.pop('ns_residual')
      self.wn = kwargs.pop('wn')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = GPSTime._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
class CarrierPhase(object):
  """CarrierPhase.
  
  Carrier phase measurement in cycles represented as a 40-bit fixed point
  number with Q32.8 layout, i.e. 32-bits of whole cycles and 8-bits of
  fractional cycles. This phase has the same sign as the pseudorange.
  
  Parameters
  ----------
  i : int
    Carrier phase whole cycles
  f : int
    Carrier phase fractional part

  """
  _parser = construct.Struct(
                     'i' / construct.Int32sl,
                     'f' / construct.Int8ul,)
  __slots__ = [
               'i',
               'f',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.i = kwargs.pop('i')
      self.f = kwargs.pop('f')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = CarrierPhase._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    

msg_classes = {
}


#!/usr/bin/env python

import os
import sys
import glob
import shutil
import platform
import tempfile
import subprocess

if 'PYPI_USERNAME' not in os.environ or 'PYPI_PASSWORD' not in os.environ:
    print("\n>>> WARNING: set PYPI_USERNAME and PYPI_PASSWORD to push to PyPI\n\n")
    sys.exit(1)

PYPI_USERNAME = os.environ.get('PYPI_USERNAME', None)
PYPI_PASSWORD = os.environ.get('PYPI_PASSWORD', None)

if 'SBP_VERSION' not in os.environ:
    print("\n!!! Please set SBP_VERSION in the environment !!!\n\n")
    sys.exit(1)

SBP_VERSION = os.environ['SBP_VERSION']

USE_TEST_PYPI = bool(os.environ.get('USE_TEST_PYPI', None))

if not platform.machine().startswith("arm") and not shutil.which('conda'):
    print("\n!!! Please install conda to deploy python !!!\n\n")
    sys.exit(1)

script_dir = os.path.dirname(os.path.abspath(__file__))
repo_dir = os.path.join(script_dir, "..")

os.chdir(script_dir)

if platform.system() == "Linux" and platform.python_version().startswith("3.4"):
    DASHDASH = ["--"]
else:
    DASHDASH = []

os.environ['IS_RELEASED'] = 'y'


def twine_upload(wheel):
    cmd_prefix = ["python3", "-m"]
    if PYPI_USERNAME is not None and PYPI_PASSWORD is not None:
        invoke = subprocess.check_call if not USE_TEST_PYPI else subprocess.call
        cmd = ["twine", "upload", "-u", PYPI_USERNAME, "-p", PYPI_PASSWORD]
        cmd = cmd_prefix + cmd + (
                ["--repository-url", "https://test.pypi.org/legacy/"] if USE_TEST_PYPI else [])
        ret_wheel = invoke(cmd + [wheel])
    else:
        print(">>> WARNING: not pushing to PyPI (one of PYPI_USERNAME or PYPI_PASSWORD was empty)")

    if USE_TEST_PYPI and ret_wheel != 0:
        print(">>> WARNING: upload of wheel failed, exit code: {}".format(ret_wheel))


def invoke_bdist():
    subprocess.check_call(["python3", "setup.py", "bdist_wheel"])


def invoke_sdist():
    subprocess.check_call(["python3", "setup.py", "sdist"])


def run_bdist(deploy_dir):
    print(">>> Building staging area for deployment ...")

    old_cwd = os.getcwd()

    os.chdir(deploy_dir)
    os.mkdir('module')

    shutil.copytree(os.path.join(repo_dir, ".git"), ".git")

    shutil.copy(os.path.join(script_dir, ".gitignore"), "module/.gitignore")
    shutil.copy(os.path.join(script_dir, ".flake8"), "module/.flake8")

    for dirent in glob.glob(os.path.join(script_dir, "*")):
        _, leaf_name = os.path.split(dirent)
        if os.path.isdir(dirent):
            print('Copying (recursive) {}'.format(dirent))
            shutil.copytree(dirent, os.path.join("module", leaf_name))
        else:
            print('Copying (non-recursive) {}'.format(dirent))
            shutil.copy(dirent, os.path.join("module", leaf_name))

    print(">>> Pruning ...")

    if os.path.exists("module/docs/_build"):
        shutil.rmtree("module/docs/_build")

    for dirent in glob.glob("module/build/*"):
        shutil.rmtree(dirent) if os.path.isdir(dirent) else os.unlink(dirent)

    os.chdir("module")

    print(">>> Staged to '{}'...'".format(deploy_dir))

    print(">>> Building Python wheel ...")

    invoke_bdist()

    whl_pattern = "dist/sbp-{}-*.whl".format(SBP_VERSION)
    print(">>> Uploading Python wheel (glob: {})...".format(whl_pattern))

    wheels = glob.glob(whl_pattern)
    if not wheels:
        print("\n!!! No Python wheel (.whl) file found...\n\n")
        sys.exit(1)

    wheel = wheels[0]

    print(">>> Found wheel (of {} matches): {}".format(len(wheels), wheel))

    print(">>> Copying wheel {} to {}".format(wheel, old_cwd))
    shutil.copy(wheel, old_cwd)

    wheel = wheel.replace("-linux_x86_64", "-manylinux1_x86_64")
    twine_upload(wheel)

    print(">>> Building source tarball ...")

    invoke_sdist()

    tarball_pattern = "dist/sbp-{}*.tar.gz".format(SBP_VERSION)
    print(">>> Uploading source tarball (glob: {})...".format(tarball_pattern))

    tarballs = glob.glob(tarball_pattern)
    if not tarballs:
        print("\n!!! No source tarball (.tgz) file found...\n\n")
        sys.exit(1)

    tarball = tarballs[0]

    print(">>> Found source tarball (of {} matches): {}".format(len(tarballs), tarball))

    print(">>> Copying tarball {} to {}".format(tarball, old_cwd))
    shutil.copy(tarball, old_cwd)

    twine_upload(tarball)


def build_wheel(deploy_dir):
    print(">>> Installing build deps in Python environment...")
    subprocess.check_call([
        "conda", "install", "--yes",
        "wheel", "setuptools", "pip"
    ])
    subprocess.check_call(
        ["python", "-m", "pip", "install", "--upgrade", "pip"])
    subprocess.check_call(
        ["python", "-m", "pip", "install", "twine"])
    print(">>> Installing setup deps in Python environment...")
    subprocess.check_call(
        ["python", "-m", "pip", "install",
         "-r", "setup_requirements.txt",
         "-r", "test_requirements.txt"])
    subprocess.check_call(
        ["python", "-m", "pip", "install",
         "-r", "requirements.txt",
         "-r", "setup_requirements.txt"])
    run_bdist(deploy_dir)


def main():
    print(">>> Building wheel...")
    deploy_dir = tempfile.mkdtemp()
    build_wheel(deploy_dir)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.


"""
Logging and debugging messages from the device.
"""

import json

import construct

from sbp.msg import SBP, SENDER_ID
from sbp.utils import fmt_repr, exclude_fields, walk_json_dict, containerize

# Automatically generated from piksi/yaml/swiftnav/sbp/logging.yaml with generate.py.
# Please do not hand edit!


SBP_MSG_LOG = 0x0401
class MsgLog(SBP):
  """SBP class for message MSG_LOG (0x0401).

  You can have MSG_LOG inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This message contains a human-readable payload string from the device
  containing errors, warnings and informational messages at ERROR, WARNING,
  DEBUG, INFO logging levels.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  level : int
    Logging level
  text : string
    Human-readable string
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'level' / construct.Int8ul,
                   'text' / construct.GreedyBytes,)
  __slots__ = [
               'level',
               'text',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgLog,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgLog, self).__init__()
      self.msg_type = SBP_MSG_LOG
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.level = kwargs.pop('level')
      self.text = kwargs.pop('text')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgLog.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgLog(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgLog._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgLog._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgLog._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgLog, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_FWD = 0x0402
class MsgFwd(SBP):
  """SBP class for message MSG_FWD (0x0402).

  You can have MSG_FWD inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This message provides the ability to forward messages over SBP.  This may
  take the form of wrapping up SBP messages received by Piksi for logging
  purposes or wrapping another protocol with SBP.

  The source identifier indicates from what interface a forwarded stream
  derived. The protocol identifier identifies what the expected protocol the
  forwarded msg contains. Protocol 0 represents SBP and the remaining values
  are implementation defined.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  source : int
    source identifier
  protocol : int
    protocol identifier
  fwd_payload : array
    variable length wrapped binary message
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'source' / construct.Int8ul,
                   'protocol' / construct.Int8ul,
                   'fwd_payload' / construct.GreedyRange(construct.Int8ul),)
  __slots__ = [
               'source',
               'protocol',
               'fwd_payload',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgFwd,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgFwd, self).__init__()
      self.msg_type = SBP_MSG_FWD
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.source = kwargs.pop('source')
      self.protocol = kwargs.pop('protocol')
      self.fwd_payload = kwargs.pop('fwd_payload')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgFwd.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgFwd(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgFwd._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgFwd._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgFwd._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgFwd, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_PRINT_DEP = 0x0010
class MsgPrintDep(SBP):
  """SBP class for message MSG_PRINT_DEP (0x0010).

  You can have MSG_PRINT_DEP inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Deprecated.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  text : string
    Human-readable string
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'text' / construct.GreedyBytes,)
  __slots__ = [
               'text',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgPrintDep,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgPrintDep, self).__init__()
      self.msg_type = SBP_MSG_PRINT_DEP
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.text = kwargs.pop('text')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgPrintDep.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgPrintDep(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgPrintDep._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgPrintDep._parser.build(c)
    return self.pack()

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgPrintDep._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgPrintDep, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    

msg_classes = {
  0x0401: MsgLog,
  0x0402: MsgFwd,
  0x0010: MsgPrintDep,
}


# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

from ...msg import SBP
from ...table import dispatch
from .base_logger import BaseLogger, LogIterator
import base64
import json
import warnings


class JSONLoggerBase(BaseLogger):
    """
    JSONLoggerBase

    The :class:`JSONLoggerBase` base class for JSON loggers.
    """

    def __init__(self, handle, tags={}, dispatcher=None, sort_keys=False):
        BaseLogger.__init__(self, handle, tags, dispatcher)
        self._sort_keys=sort_keys

    def fmt_msg(self, data, **metadata):
        if metadata:
            metadata.update(self.tags)
            metadata['data'] = data
            return metadata

        return data

    def dump(self, msg, **metadata):
        raise NotImplementedError

    def __call__(self, msg, **metadata):
        output = self.dump(msg, **metadata)
        if output:
            self.handle.write(output + "\n")


class JSONLogger(JSONLoggerBase):
    """
    JSONLogger

    The :class:`JSONLogger` logs JSON records.
    """

    def dump(self, msg, **metadata):
        try:
            data = self.dispatch(msg).to_json_dict()
            return json.dumps(self.fmt_msg(data, **metadata),
                              allow_nan=False,
                              sort_keys=self._sort_keys,
                              separators=(',',':'))
        except (ValueError, UnicodeDecodeError):
            try:
                warn = "Bad values in JSON encoding for msg_type %d for msg %s" \
                       % (msg.msg_type, msg)
                warnings.warn(warn, RuntimeWarning)
                return json.dumps(self.fmt_msg(msg.to_json_dict(), **metadata))
            except (ValueError, UnicodeDecodeError):
                return None


class JSONBinLogger(JSONLoggerBase):
    """
    JSONBinLogger

    The :class:`JSONLogger` logs JSON records without expanding the fields.
    """

    def dump(self, msg, **metadata):
        try:
            data = {
                'preamble': msg.preamble,
                'msg_type': msg.msg_type,
                'sender': msg.sender,
                'length': msg.length,
                'payload': base64.standard_b64encode(msg.payload).decode('ascii'),
                'crc': msg.crc
            }
            return json.dumps(self.fmt_msg(data, **metadata),
                              allow_nan=False,
                              sort_keys=self._sort_keys,
                              separators=(',',':'))
        except (ValueError, UnicodeDecodeError):
            try:
                warn = "Bad values in JSON encoding for msg_type %d for msg %s" \
                       % (msg.msg_type, msg)
                warnings.warn(warn, RuntimeWarning)
                return json.dumps(self.fmt_msg(msg.to_json_dict(), **metadata))
            except (ValueError, UnicodeDecodeError):
                return None


class JSONLogIterator(LogIterator):
    """
    JSONLogIterator

    The :class:`JSONLogIterator` is an iterator for reading JSON logs
    of SBP data.

    Parameters
    ----------
    filename : string
      Path to file to read SBP messages from.
    """

    def __init__(self, handle, dispatcher=dispatch, conventional=False):
        super(JSONLogIterator, self).__init__(handle, dispatcher=dispatcher)
        self._broken = False
        self._conventional = conventional

    def breakiter(self):
        """
        Break out of message iteration.
        May be called from another thread when some thread is blocked in next().
        """
        self._broken = True

    def _extract_data(self, line):
        data = json.loads(line)
        if "payload" in data.keys():
            item = SBP.from_json_dict(data)
        else:
            item = SBP.from_json_dict(data.pop('data', data))
        msg = self.dispatch(item, line)
        return (msg, data)

    def _next_conventional(self):
        if self._broken:
            raise StopIteration

        line = ''
        try:
            line = self.handle.readline()

            if not line:
                raise StopIteration

            return self._extract_data(line)
        except (ValueError, UnicodeDecodeError):
            warn = "Bad JSON decoding for line %s" % line
            warnings.warn(warn, RuntimeWarning)
            # Try next line, can lead to RecursionError if source is full of rubbish
            return self._next_conventional()

    def _next_legacy_inner(self):
        line = ''
        for line in self.handle:
            try:
                yield self._extract_data(line)
            except (ValueError, UnicodeDecodeError):
                warn = "Bad JSON decoding for line %s" % (line,)
                warnings.warn(warn, RuntimeWarning)

    def _next_legacy(self):
        try:
            for line in self._next_legacy_inner():
                yield line
        except (ValueError, UnicodeDecodeError):
            warn = "Bad JSON decoding"
            warnings.warn(warn, RuntimeWarning)
        self.handle.seek(0, 0)

    def __next__(self):
        """
        Return the next record tuple from log file containing
        JSON-serialized SBP. If an unknown SBP message type is found,
        it'll return the raw SBP. If EOF, throws exception and then
        returns to start of file.

        Returns
        -------
        Tuple(sbp MSG object, {'time':'ISO 8601 time'})
          Second item is for metadata. There used to be multiple fields and
          there could be more in the future.

        Notes
        -----
        In practice it seems like a lot of times JSONLogIterator objects return
        iterators instead of tuples due to weird usage of the class.

        """
        if self._conventional:
            return self._next_conventional()
        else:
            return self._next_legacy()



#!/usr/bin/env python
# Copyright (C) 2011-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

"""
Single dispatch of available SBP messages, keyed by msg_type.

"""

from construct.core import FormatFieldError
from . import acquisition as acq
from . import bootload as boot
from . import file_io as file_io
from . import flash as flash
from . import linux as linux
from . import logging as log
from . import navigation as nav
from . import observation as obs
from . import piksi as piksi
from . import settings as settings
from . import system as sys
from . import tracking as trac
from . import ext_events as ext_events
from . import user as user
from . import imu as imu
from . import integrity as integrity
from . import mag as mag
from . import ndb as ndb
from . import vehicle as vehicle
from . import orientation as orientation
from . import sbas as sbas
from . import ssr as ssr
from . import solution_meta as solmeta
from . import signing as signing

import warnings

_SBP_TABLE = dict(list(acq.msg_classes.items())
                  + list(boot.msg_classes.items())
                  + list(file_io.msg_classes.items())
                  + list(flash.msg_classes.items())
                  + list(linux.msg_classes.items())
                  + list(log.msg_classes.items())
                  + list(nav.msg_classes.items())
                  + list(obs.msg_classes.items())
                  + list(piksi.msg_classes.items())
                  + list(settings.msg_classes.items())
                  + list(sys.msg_classes.items())
                  + list(trac.msg_classes.items())
                  + list(user.msg_classes.items())
                  + list(imu.msg_classes.items())
                  + list(integrity.msg_classes.items())
                  + list(mag.msg_classes.items())
                  + list(ext_events.msg_classes.items())
                  + list(ndb.msg_classes.items())
                  + list(vehicle.msg_classes.items())
                  + list(orientation.msg_classes.items())
                  + list(sbas.msg_classes.items())
                  + list(ssr.msg_classes.items())
                  + list(solmeta.msg_classes.items())
                  + list(signing.msg_classes.items())
                  )

class InvalidSBPMessageType(NotImplementedError):
  """
  Base exception for messages with invalid message types.
  """

def dispatch(msg, table=_SBP_TABLE):
  """
  Dispatch an SBP message type based on its `msg_type` and parse its
  payload.

  Parameters
  ----------
  driver : :class:`SBP`
    A parsed SBP object.
  table : dict
    Any table mapping unique SBP message type IDs to SBP message
    constructors.

  Returns
  ----------
  SBP message with a parsed payload.

  """
  try:
    return table[msg.msg_type](msg)
  except KeyError:
    warn = "No message found for msg_type id %d for msg %s." \
           % (msg.msg_type, msg)
    warnings.warn(warn, RuntimeWarning)
    return msg
  except FormatFieldError:
    warnings.warn("SBP payload deserialization error! 0x%x" % msg.msg_type,
                  RuntimeWarning)
    return msg

def lookup(msg_type, table=_SBP_TABLE):
    """
    Finds the SBP message class according to the message id given

    Parameters
    ----------
    msg_type : Int
      Type of the message
    table : dict
      Any table mapping unique SBP message type IDs to SBP message
      constructors.

    Returns
    ----------
    SBP Message class

    """
    return table.get(msg_type)



# Copyright (C) 2016-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

from .base_driver import BaseDriver


class CdcDriver(BaseDriver):
    """
    CdcDriver
    The :class:`CdcDriver` class wraps IO sources of SBP messages and provides
    context management.  It is intended for the devices that use the USB Gadget
    CDC ACM drivers and is preferred over PySerial for these devices.

    Parameters
    ----------
    handle : port
      Stream of bytes to read from and write to.
    """

    def _read(self, size):
        """
        Read wrapper.

        Parameters
        ----------
        size : int
          Number of bytes to read.
        """
        try:
            return_val = self.handle.read(size)
            if return_val == '':
                print()
                print("Piksi disconnected")
                print()
                self.close()
                raise IOError
            return return_val
        except OSError:
            print()
            print("Piksi disconnected")
            print()
            self.close()
            raise IOError

    def _write(self, s):
        """
        Write wrapper.

        Parameters
        ----------
        s : bytes
          Bytes to write
        """
        try:
            return self.handle.write(s)
        except OSError:
            print()
            print("Piksi disconnected")
            print()
            self.close()
            raise IOError

    def close(self):
        """
        Close wrapper.
        """
        try:
            self.handle.close()
        except (OSError, IOError):
            pass
